#Semiparametric estimation applied to the joint retirement problem including 'age difference' as the third regressor
library('KernSmooth');
library('locfit');
library('purrr');
library('pracma');

library('readxl');
#Use the HRS data in Honore and Paula 2018
hpdatamata_excel <- read_excel("C:/replicationfiles_HonorePaula_QE/Matlab/hpdatamata_excel.xls");
attach(hpdatamata_excel);

data<-hpdatamata_excel[c(1:6,50,62)];
attach(data);
Z1<-rtothealthexpen[gender==1];
Z2<-rtothealthexpen[gender==0];

Ft<-failure;
n<-length(Ft);
ID<-seq(1,n,1);
Ft1<-Ft[!ID%%2==0];
Ft2<-Ft[ID%%2==0];
D<-rep(1,n/2);
for (i in 1:(n/2)){
  if (Ft1[i]<Ft2[i]){
    D[i]<-1
  }
  else if (Ft1[i]==Ft2[i]){
    D[i]<-3
  }
  else if (Ft1[i]>Ft2[i]){
    D[i]<-2
  }
}

S1<-rep(1,n/2);
for (i in 1:(n/2)){
  if (abs(Ft1[i]-Ft2[i])<2){
    S1[i]<-1
  }
  else S1[i]<-0
}
S2<-rep(1,n/2);
for (i in 1:(n/2)){
  if (abs(Ft1[i]-Ft2[i])<3){
    S2[i]<-1
  }
  else S2[i]<-0
}
#Create minimum duration V within the household
hpdata<-aggregate(data,by=list(hh),FUN=min,na.rm=T);
health<-aggregate(data,by=list(hh),FUN=mean,na.rm=T);
TotalExp<-health$rtothealthexpen;
hpdatac<-cbind(hpdata,D,TotalExp,Z1,Z2,S1,S2);
#Exclude the obs. with the minimum duration being censored
hpdata_uncensored<-hpdatac[which(hpdatac$censored==0),]
#Exclude the obs. with age difference recorded as 'Inf'
hpdata_final<-hpdata_uncensored[which(!hpdata_uncensored$TotalExp=='NaN'),]
hpdata_final2<-hpdata_final[which(!hpdata_final$agedif=='Inf'),]

#Collect the competing risks data and covariates from the dataset
X1<-hpdata_final2$TotalExp;
X2<-hpdata_final2$fweal;
X3<-hpdata_final2$agedif;
X3<-(X3-mean(X3))/sd(X3); #Standardize the age difference
V<-hpdata_final2$failure;
D<-hpdata_final2$D;
n<-length(V);

V<-log(V); #Log-transform of the duration

#Bandwidth used
h1<-dpik(X1);
h2<-dpik(X2);
h3<-dpik(X3);
hv=dpik(V);

#Kernel Estimators of Conditional Survivor Function of the minimum duration and Sub-Densities



#Kernel Weight
Bk2.func = function(x1,X1,h1,x2,X2,h2,x3,X3,h3)
{
  # Kernel Weights using Bisquare Kernel function
  # h is the bandwidth
  xx_1<-(X1-x1)/h1;  
  xx_1[abs(xx_1)>=1]<-1;
  xx_2<-(X2-x2)/h2;
  xx_2[abs(xx_2)>=1]<-1
  K_1<-15*(1-xx_1^2)^2/16;  #biquadratic kernel 
  K_2<-15*(1-xx_2^2)^2/16;
  xx_3<-(X3-x3)/h3;
  xx_3[abs(xx_3)>=1]<-1
  K_3<-15*(1-xx_3^2)^2/16;  #biquadratic kernel 
  K<-K_1*K_2*K_3;
  W<-K/sum(K);
  return(W);
}
Kern.func = function(u0, U, h)
{
  # the kernel weight function Kern(u0, U) returns a vector of kernel weights used in the smoothed MLE estimates,
  #where u0 is a scalar, and U is a vector returns a vector
  # h is the bandwidth
  uu<-(U-u0)/h;  
  uu[abs(uu)>=1]<-1;
  r<-15*(1-uu^2)^2/16;  #biquadratic kernel is used here
  return(r)
}

Sv2.func=function(t,x1,x2,x3,V,X1,X2,X3,h1,h2,h3){
  return(as.numeric(V>t)%*%Bk2.func(x1,X1,h1,x2,X2,h2,x3,X3,h3))
}


Sv2Dj.func=function(t,x1,x2,x3,j,V,X1,X2,X3,h1,h2,h3){
  return(as.numeric(V>t & D==j)%*%Bk2.func(x1,X1,h1,x2,X2,h2,x3,X3,h3))
}


fv2.func=function(t,x1,x2,x3,V,X1,X2,X3,h1,h2,h3,hv){
  return(hv^(-1)*Kern.func(t,V,hv)%*%Bk2.func(x1,X1,h1,x2,X2,h2,x3,X3,h3))
}

fvDj.func=function(t,x1,x2,x3,j,V,X1,X2,X3,D,h1,h2,h3,hv){
  return(hv^(-1)*(Kern.func(t,V,hv)[D==j]%*%Bk2.func(x1,X1,h1,x2,X2,h2,x3,X3,h3)[D==j]))
  
}

r_hat<-rep(1,n);
s_hat<-rep(1,n);

for (i in 1:n) {
  r_hat[i]<-1-Sv2.func(V[i],X1[i],X2[i],X3[i],V,X1,X2,X3,h1,h2,h3);
  s_hat[i]<-fv2.func(V[i],X1[i],X2[i],X3[i],V,X1,X2,X3,h1,h2,h3,hv);
}

fn.1<-rep(1,n);
fn.2<-rep(1,n);
for (i in 1:n) {
  fn.1[i]<-fvDj.func(V[i],X1[i],X2[i],X3[i],j=1,V,X1,X2,X3,D,h1,h2,h3,hv);
  fn.2[i]<-fvDj.func(V[i],X1[i],X2[i],X3[i],j=2,V,X1,X2,X3,D,h1,h2,h3,hv);
}


Wtn<-dnorm(X1-mean(X1))*dnorm(X2-mean(X2))*dnorm(X3);


#kNN based nonparametric estimation

k=ceiling(n^(2/5));

q_func<-function(t,z0){
  R<-r_hat-t;
  V0<-V-z0;
  Dt<-sqrt(R^2+V0^2);
  
  Os_hat<-s_hat[order(Dt)];
  Os_hat_k<-Os_hat[1:k];
  
  return(mean(Os_hat_k))
}


#Numerical Integration by trapezoid rule
trapezoid<-function(f,a,b,l=20){
  points<-seq(a,b,length=l+1)
  g<-(b-a)/l
  area<-0
  for (i in seq_len(l)) {
    area<-area+g/2*(f(points[i])+f(points[i+1]))
    }
  area
}
#Estimation of the multiplicative model in Jacho-Chavez et al.

M2.func<-function(x1,x2,x3,z){
  r_hat.xz<-Sv2.func(z,x1,x2,x3,V,X1,X2,X3,h1,h2,h3);
  r1<-1;
  q_hat.uz<-function(u){
    out<-q_func(u,z0=mean(V));
    return(out);
  }
  out=exp(trapezoid(q_hat.uz,a=min(r1,r_hat.xz),b=max(r1,r_hat.xz),l=20) );
  return(out)
}


alphax2.func<-function(x1,x2,x3){
  M_nz.func=Vectorize(M2.func,'z')
  out=mean(M_nz.func(x1,x2,x3,V));
  return(out)
}


alphaz2.func<-function(z){
  #Use map2 function to handle bivariate X.
  M_nx.func=function(x1,x2,x3){
    return(M2.func(x1,x2,x3,z))
  }
  
  out=mean(as.numeric(pmap(list(X1,X2,X3),M_nx.func)));
  return(out)
}

b_tilde<-as.numeric(pmap(list(X1,X2,X3,V),M2.func));

c_tilde<-mean(as.numeric(b_tilde) );

G_tilde.func<-function(x1,x2,x3){
  out=alphax2.func(x1,x2,x3)/c_tilde;
  return(out)
}
F_tilde.func<-function(z){
  out=alphaz2.func(z)/c_tilde;
  return(out)
}


Gn_tilde<-as.numeric(pmap(list(X1,X2,X3),G_tilde.func));

Fn_tilde<-sapply(V, F_tilde.func);

Mn_tilde<-Gn_tilde*Fn_tilde*c_tilde;


VO<-V[order(V)];
Fv<-Fn_tilde;
FvO<-Fv[order(V)];


h0=dpill(Fn_tilde*Gn_tilde,1-r_hat);

fit0<-locpoly(Mn_tilde,1-r_hat,drv=0,gridsize = n,bandwidth =3*h0);
fit1<-locpoly(Fn_tilde*Gn_tilde,1-r_hat,drv=1,gridsize=n,bandwidth=3*h0);

rho<-0.01; #Trimming Parameter
deno<-pmax(rho,abs(fit1$y))*sign(fit1$y);

#Regression Coefficient for the Wife
#Coordinate 1
gamma_11=mean(X1*Wtn*Gn_tilde)-mean(X1*Wtn*fn.2/deno);
#Coordinate 2
gamma_12=mean(X2*Wtn*Gn_tilde)-mean(X2*Wtn*fn.2/deno);
#Coordinate 3
gamma_13=mean(X3*Wtn*Gn_tilde)-mean(X3*Wtn*fn.2/deno);
#Normalization
norm1<-sqrt(gamma_11^2+gamma_12^2+gamma_13^2);
gamma.n_11<-(gamma_11)/norm1;
gamma.n_12<-gamma_12/norm1;
gamma.n_13<-gamma_13/norm1

#Regression Coefficient for the Husband
#Coordinate 1
gamma_21=mean(X1*Wtn*Gn_tilde)-mean(X1*Wtn*fn.1/deno);
#Coordinate 2
gamma_22=mean(X2*Wtn*Gn_tilde)-mean(X2*Wtn*fn.1/deno);
#Coordinate 3
gamma_23=mean(X3*Wtn*Gn_tilde)-mean(X3*Wtn*fn.1/deno);
#Normalization
norm2<-sqrt(gamma_21^2+gamma_22^2+gamma_23^2);
gamma.n_21<-(gamma_21)/norm2;
gamma.n_22<-gamma_22/norm2;
gamma.n_23<-gamma_23/norm2;

